#!/usr/bin/env python
# coding: utf-8
#
#       AUTHORS
#       Lachlan de Waard <lachlan.00@gmail.com>
#       Wolter Hellmund <wolterh6@gmail.com>
#       --------------------------------------
#       Fileorganizer file operations
#       --------------------------------------
#       Creative Commons - Attribution Share Alike v3.0

import os
import shutil
import urllib
import time
import ConfigParser
import eyeD3
import mimetypes

from gi.repository import RB

import tools

from logops import LogFile
from dbops import UrlData


RB_METATYPES = ('at', 'aa', 'aA', 'as', 'aS', 'ay', 'an', 'aN', 'ag', 'aG',
                'tn', 'tN', 'tt', 'ta', 'tA')
RB_COVER_CACHE = '/.cache/rhythmbox/covers/'
RB_MEDIA_TYPES = ['.m4a', '.flac', '.ogg', '.mp2', '.mp3', '.wav', '.spx']

PROP = [RB.RhythmDBPropType.ALBUM, RB.RhythmDBPropType.ALBUM_ARTIST,
        RB.RhythmDBPropType.ALBUM_ARTIST_FOLDED,
        RB.RhythmDBPropType.ALBUM_ARTIST_SORTNAME,
        RB.RhythmDBPropType.ALBUM_ARTIST_SORTNAME_FOLDED,
        RB.RhythmDBPropType.YEAR, RB.RhythmDBPropType.DISC_NUMBER,
        RB.RhythmDBPropType.GENRE, RB.RhythmDBPropType.GENRE_FOLDED,
        RB.RhythmDBPropType.TRACK_NUMBER, RB.RhythmDBPropType.TITLE,
        RB.RhythmDBPropType.ARTIST, RB.RhythmDBPropType.ARTIST_FOLDED]

IN = '    IN:    '
OUT = '    OUT:   '
INFO = ' ** INFO: '
ERROR = ' ** ERROR: '
CONFLICT = ' ** CONFLICT: '
NO_NEED = 'No need for file relocation'
STILL_MEDIA = 'Directory still contains media; keeping:'
FILE_EXISTS = 'File exists, directing to backup folder'
POSSIBLE_DAMAGE = "Source file damaged or missing tag information.\n"
ART_MOVED = 'Cover art moved to cache folder'
DIR_REMOVED = 'Removed empty directory'
UPDATING = 'Updating Database:'

previewlist = []


class MusicFile():

    def __init__(self, fileorganizer, db_entry=None):
        self.conf = ConfigParser.RawConfigParser()
        conffile = (os.getenv('HOME') + '/.local/share/rhythmbox/' +
                         'plugins/fileorganizer-gtk3/fo.conf')
        self.conf.read(conffile)
        self.fo = fileorganizer
        self.db = self.fo.db
        if db_entry:
            # Track and disc digits from gconf
            padded = '%s' % ('%0' + str(2) + '.d')
            single = '%s' % ('%0' + str(1) + '.d')
            self.metadata = {
                RB_METATYPES[0]: db_entry.get_string(PROP[0]),
                RB_METATYPES[1]: db_entry.get_string(PROP[1]),
                RB_METATYPES[2]: db_entry.get_string(PROP[2]),
                RB_METATYPES[3]: db_entry.get_string(PROP[3]),
                RB_METATYPES[4]: db_entry.get_string(PROP[4]),
                RB_METATYPES[5]: str(db_entry.get_ulong(PROP[5])),
                RB_METATYPES[6]: str(single % (db_entry.get_ulong(PROP[6]))),
                RB_METATYPES[7]: str(padded % (db_entry.get_ulong(PROP[6]))),
                RB_METATYPES[8]: db_entry.get_string(PROP[7]),
                RB_METATYPES[9]: db_entry.get_string(PROP[8]),
                RB_METATYPES[10]: str(single % (db_entry.get_ulong(PROP[9]))),
                RB_METATYPES[11]: str(padded % (db_entry.get_ulong(PROP[9]))),
                RB_METATYPES[12]: db_entry.get_string(PROP[10]),
                RB_METATYPES[13]: db_entry.get_string(PROP[11]),
                RB_METATYPES[14]: db_entry.get_string(PROP[12])
            }
            self.location = db_entry.get_string(RB.RhythmDBPropType.LOCATION)
            self.entry = db_entry

    # Returns metadata of the music file
    def get_metadata(self, key):
        for datum in self.metadata:
            if key == datum:
                return self.metadata[datum]

    # Rhythmbox coverart
    def set_rb_coverart(self, source):
        coverart_enabled = self.conf.get('conf', 'cover_enabled')
        coverart_names = self.conf.get('conf', 'cover_names')
        coverart_names = coverart_names.split(',')
        if coverart_enabled == 'True':
            artfile = '%ta - %at'
            artfile = (tools.data_filler(self, artfile) + '.jpg')
            artfile = os.getenv('HOME') + RB_COVER_CACHE + artfile
            if not os.path.isfile(artfile):
                print 'COVERART MISSING'
                for filenames in coverart_names:
                    test = os.path.dirname(source) + '/' + filenames
                    if os.path.isfile(test):
                        print 'COPYING COVERART TO RB CACHE'
                        self.log.log_processing(INFO + ART_MOVED)
                        shutil.copyfile(test, artfile)

    # Non media clean up
    def file_cleanup(self, source, destin):
        cleanup_enabled = self.conf.get('conf', 'cleanup_enabled')
        remove_folders = self.conf.get('conf', 'cleanup_empty_folders')
        if cleanup_enabled == 'True':
            sourcedir = os.path.dirname(source)
            destindir = os.path.dirname(destin)
            foundmedia = False
            # Remove empty folders, if any
            if os.path.isdir(sourcedir):
                if os.listdir(sourcedir) == [] and remove_folders == 'True':
                    print '%s (directory) now empty; removing' % sourcedir
                    self.log.log_processing(INFO + DIR_REMOVED)
                    self.log.log_processing('    ' + sourcedir)
                    os.rmdir(sourcedir)
                else:
                    for files in os.listdir(sourcedir):
                        filelist = files[(files.rfind('.')):]
                        if filelist in RB_MEDIA_TYPES or os.path.isdir(
                                sourcedir + '/' + files):
                            foundmedia = True
                        elif not destindir == sourcedir:
                            mvdest = destindir + '/' + os.path.basename(files)
                            mvsrc = sourcedir + '/' + os.path.basename(files)
                            try:
                                shutil.move(mvsrc, mvdest)
                                self.log.log_processing(INFO + 'Moved')
                                self.log.log_processing('    ' + mvdest)
                            except:
                                self.log.log_processing(ERROR + mvdest)
                                pass
                    if foundmedia == True:
                        self.log.log_processing(INFO + STILL_MEDIA)

    # Get Source and Destination seperately so preview can use the same code
    def get_locations(self, inputString):
        # Get source for comparison
        source = urllib.url2pathname(self.location).replace('file:///', '/')
        if inputString == 'source':
            return source
        # Set Destination Directory
        targetdir = '/' + self.fo.configurator.get_val('layout-path')
        targetdir = tools.data_filler(self, targetdir)
        targetdir = tools.folderize(self.fo.configurator, targetdir)
        # Set Destination  Filename
        targetname = self.fo.configurator.get_val('layout-filename')
        targetname = tools.data_filler(self, targetname)
        targetname += os.path.splitext(self.location)[1]
        # Join destination
        destin = (os.path.join(targetdir, targetname)).replace('file:///', '/')
        if inputString == 'destin':
            return destin
        return

    #tag operations
    def update_tags(self, inputfile):
        update_tags = self.conf.get('conf', 'update_tags')
        if update_tags == 'True':
            self.log.log_processing('          Updating File Tags')
            t = eyeD3.Tag()
            if mimetypes.guess_type(inputfile)[0] == 'audio/mpeg':
                print 'UPDATING TAGS'
                t.link(inputfile)
                t.setArtist(self.entry.get_string(RB.RhythmDBPropType.ARTIST))
                t.setArtist(self.entry.get_string(
                                    RB.RhythmDBPropType.ALBUM_ARTIST), 'TPE2')
                t.setTrackNum([str(self.entry.get_ulong(
                                    RB.RhythmDBPropType.TRACK_NUMBER)), None])
                t.setDiscNum([str(self.entry.get_ulong(
                                    RB.RhythmDBPropType.DISC_NUMBER)), None])
                t.setDate(str(self.entry.get_ulong(
                                    RB.RhythmDBPropType.YEAR)), 0)
                t.setAlbum(self.entry.get_string(RB.RhythmDBPropType.ALBUM))
                t.setTitle(self.entry.get_string(RB.RhythmDBPropType.TITLE))
                t.setGenre(self.entry.get_string(RB.RhythmDBPropType.GENRE))
                t.update(eyeD3.ID3_V2_4)
        return

    def check_bad_file(self, inputFile):
        # Only check MP3's
        if not mimetypes.guess_type(inputFile)[0] == 'audio/mpeg':
            return True
        t = eyeD3.Tag()
        t.link(inputFile)
        #fail on important tags
        if t.getArtist() == '' or t.getTitle() == '' or t.getAlbum() == '':
            return False
        if str(t.getYear()) == 'None':
            try:
                if t.getDate(0)[0].text == '':
                    return False
            except:
                return False
        return True

    def preview(self):
        print 'preview'
        previewlist = os.getenv('HOME') + '/.fileorganizer-preview.log'
        damagedlist = os.getenv('HOME') + '/.fileorganizer-damaged.log'
        source = self.get_locations('source')
        destin = self.get_locations('destin')
        if not self.check_bad_file(source):
                FILE = open(damagedlist, "a")
                FILE.write(POSSIBLE_DAMAGE + source + "\n")
                FILE.write("File Size:  " + str(os.stat(source)[6] / 1024)
                            + "kb\n\n")
        elif not source == destin:
                # Write to preview list
                FILE = open(previewlist, "a")
                FILE.write("Change Found:\n" + source + "\n")
                FILE.write(destin + "\n\n")
                FILE.close()

    # Moves the file to a specific location with a specific name
    def relocate(self):
        source = self.get_locations('source')
        destin = self.get_locations('destin')
        # Begin Log File
        CURRENT_TIME = time.strftime("%I:%M:%S %p", time.localtime())
        logheader = '%ta - %at - '
        logheader = (tools.data_filler(self, logheader) + CURRENT_TIME)
        self.log = LogFile()
        self.log.log_processing(logheader)
        self.log.log_processing((IN + source))
        # Save cover art if found into Rhythmbox cover cache
        self.set_rb_coverart(source)

        # Relocate, if necessary
        if source == destin:
            print 'No need for file relocation'
            self.log.log_processing(INFO + NO_NEED)
        else:
            if os.path.isfile(destin):
                # Copy the existing file to a backup dir
                backupdir = (((self.fo.configurator.get_val('locations'))[0]
                               + '/backup/').replace('file:///', '/'))
                backup = os.path.join(backupdir, os.path.basename(destin))
                if os.path.isfile(backup):
                    counter = 0
                    backuptest = backup
                    while os.path.isfile(backup):
                        backup = backuptest
                        counter = counter + 1
                        backup = (backup[:(backup.rfind('.'))] + str(counter)
                                  + backup[(backup.rfind('.')):])
                try:
                    os.makedirs(os.path.dirname(backupdir))
                except:
                    pass
                destin = backup
                self.log.log_processing(CONFLICT + FILE_EXISTS)
            print 'source   ' + source
            print 'destin   ' + destin
            try:
                mvsource = unicode(source, 'utf-8')
                mvdestin = unicode(destin, 'utf-8')
                shutil.move(mvsource, mvdestin)
            except:
                self.log.log_processing(ERROR + 'moving file')
                pass
            self.log.log_processing(OUT + destin)

            # Update Rhythmbox database
            self.url = UrlData()
            self.location = self.url.set_ascii(urllib.pathname2url(destin))
            self.location = ('file://' + self.location)
            print 'Relocating file %s to %s' % (source, destin)
            self.log.log_processing(INFO + UPDATING)
            # Make the change
            self.db.entry_set(self.entry,
                              RB.RhythmDBPropType.LOCATION, self.location)
            self.log.log_processing(OUT + self.location)
            self.update_tags(destin)
            # Non media clean up
            self.file_cleanup(mvsource, mvdestin)
        self.log.log_processing('')
